package org.openspaces.example.helloworld.processor;

import com.j_spaces.core.client.SQLQuery;
import org.openspaces.events.EventTemplate;
import org.openspaces.events.adapter.SpaceDataEvent;
import org.openspaces.events.polling.Polling;
import org.openspaces.example.helloworld.common.Message;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;



/*
 * The processor is passed interesting Objects from its associated PollingContainer
 * <p>The PollingContainer removes objects from the GigaSpace that match the criteria
 * specified for it.
 * <p>Once the Processor receives each Object, it modifies its state and returns it to
 * the PollingContainer which writes them back to the GigaSpace
 * <p>
 * <p>The PollingContainer is configured in the pu.xml file of this project
 */
@Polling
@Transactional
public class Processor {
    Logger logger = LoggerFactory.getLogger(this.getClass());

    @Value("${template.value:Hello}")
    private String templateValue ;

    public Processor() {
        logger.info("Processor instantiated.");
    }

    @EventTemplate
    public SQLQuery<Message> getSqlQuery(){
        logger.info("Processing messages with info='{}'",templateValue);
        return new SQLQuery<>(Message.class,"info = ?").setParameter(1, templateValue);
    }

    /*
     * Process the given Message and return it to the caller.
     * This method is invoked using OpenSpaces Events when a matching event
     * occurs.
     */
    @SpaceDataEvent
    public Message processMessage(Message msg) {
        logger.info("Processor PROCESSING: " + msg);
        msg.setInfo(msg.getInfo() + " World !!");
        return msg;
    }
}
